/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.dnd;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.target.*;

/**
 * The <code>TargetResourceTransfer</code> class is used to transfer an
 * array of <code>ITargetResources</code>s from one part to another in a 
 * drag and drop operation or a cut, copy, paste action.
 * <p>
 * In every drag and drop operation there is a <code>DragSource</code> and 
 * a <code>DropTarget</code>.  When a drag occurs a <code>Transfer</code> is 
 * used to marshall the drag data from the source into a byte array.  If a drop 
 * occurs another <code>Transfer</code> is used to marshall the byte array into
 * drop data for the target.  
 * </p>
 * <p>
 * When a <code>CutAction</code> or a <code>CopyAction</code> is performed, 
 * this transfer is used to place references to the selected resources 
 * on the <code>Clipboard</code>.  When a <code>PasteAction</code> is performed, the 
 * references on the clipboard are used to move or copy the resources
 * to the selected destination.
 * </p>
 * <p>
 * This class can be used for a <code>Viewer<code> or an SWT component directly.
 * A singleton is provided which may be serially reused (see <code>getInstance</code>).  
 * It is not intended to be subclassed.
 * </p>
 *
 * @see org.eclipse.jface.viewers.StructuredViewer
 * @see org.eclipse.swt.dnd.DropTarget
 * @see org.eclipse.swt.dnd.DragSource
 */
public class TargetResourceTransfer extends ByteArrayTransfer {

    //Singleton instance.
    private static final TargetResourceTransfer instance = new TargetResourceTransfer();

    // Create a unique ID to make sure that different Eclipse
    // applications use different "types" of <code>ResourceTransfer</code>
    private static final String TYPE_NAME = "target-resource-transfer-format:" + System.currentTimeMillis() + ":" + instance.hashCode();//$NON-NLS-2$//$NON-NLS-1$
    private static final int TYPEID = registerType(TYPE_NAME);
    
    /**
     * Returns the singleton instance.
     *
     * @return the singleton instance
     */
    public static TargetResourceTransfer getInstance() {
        return instance;
    }
    
	/*
	 * Allow only one instance.
	 * @see #getInstance()
	 */
	private TargetResourceTransfer() {
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.Transfer#getTypeIds()
	 */
	protected int[] getTypeIds() {
		return new int[] { TYPEID };
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.Transfer#getTypeNames()
	 */
	protected String[] getTypeNames() {
		return new String[] { TYPE_NAME };
	}
	
    /* (non-Javadoc)
     * Method declared on Transfer.
     */
    protected void javaToNative(Object data, TransferData transferData) {
        if (!(data instanceof ITargetResource[])) {
            return;
        }
        byte[] bytes = toByteArray((ITargetResource[]) data);
        if (bytes != null)
        	super.javaToNative(bytes, transferData);
    }

	/* (non-Javadoc)
     * Method declared on Transfer.
     */
    protected Object nativeToJava(TransferData transferData) {
        byte[] bytes = (byte[]) super.nativeToJava(transferData);
        return fromByteArray(bytes);
    }

    /**
     * Marshall the given resources into a byte array.
	 * @param resources the resources to be serialized
	 * @return the byte array or <code>null</code> if problems where encountered
	 */
	public byte[] toByteArray(ITargetResource[] resources) {
        /**
         * The target resource serialization format is:
         *  (int) number of resources
         * Then, the following for each resource:
         *  (String) the site location identifier
         *  (boolean) whther the resource is a directory
         *  (String) the site relative path of resource
         */
		int resourceCount = resources.length;

        byte[] bytes = null;
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            DataOutputStream dataOut = new DataOutputStream(out);

            //write the number of resources
            dataOut.writeInt(resourceCount);

            //write each resource
            for (int i = 0; i < resources.length; i++) {
                writeResource(dataOut, resources[i]);
            }

            //cleanup
            dataOut.close();
            out.close();
            bytes = out.toByteArray();
        } catch (IOException e) {
            //it's best to send nothing if there were problems
        }
		return bytes;
	}
	
    /**
     * Return the target resource that are contained in the marshalled byte array
     * or <code>null</code> if there are none.
	 * @param bytes the marshalled bytes
	 * @return the target resource
	 */
	public ITargetResource[] fromByteArray(byte[] bytes) {
        /**
         * The target resource serialization format is:
         *  (int) number of resources
         * Then, the following for each resource:
         *  (String) the site location identifier
         *  (boolean) whther the resource is a directory
         *  (String) the site relative path of resource
         */
		if (bytes == null)
            return null;
        DataInputStream in = new DataInputStream(
                new ByteArrayInputStream(bytes));
        try {
            int count = in.readInt();
            List result = new ArrayList();
            for (int i = 0; i < count; i++) {
                ITargetResource resource = readResource(in);
                if (resource != null) {
                	result.add(resource);
                }
            }
            return (ITargetResource[]) result.toArray(new ITargetResource[result.size()]);
        } catch (IOException e) {
            return null;
        }
	}

	/**
     * Writes the given resource to the given stream.
     *
     * @param dataOut the output stream
     * @param resource the resource
     * @exception IOException if there is a problem writing to the stream
     */
    private void writeResource(DataOutputStream dataOut, ITargetResource resource) throws IOException {
        dataOut.writeUTF(resource.getSite().getLocation());
        dataOut.writeBoolean(resource.isDirectory());
        dataOut.writeUTF(resource.getRelativePath());
    }
    
    /**
     * Reads a resource from the given stream.
     *
     * @param dataIn the input stream
     * @return the resource or <code>null</code> if the resource annot be accessed
     * @exception IOException if there is a problem reading from the stream
     */
    private ITargetResource readResource(DataInputStream dataIn) throws IOException {
        String siteLocation = dataIn.readUTF();
        boolean isDirectory = dataIn.readBoolean();
        String siteRelativePath = dataIn.readUTF();
        Site site = Target.getSiteManager().getSite(siteLocation);
        if (site == null) {
        	// The site for the resource no longer exists so just skip it
        	return null;
        }
        if (isDirectory) {
        	return site.getRootResource().getFolder(siteRelativePath);
        } else {
        	return site.getRootResource().getFile(siteRelativePath);
        }
    }
}
