/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.webdav;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.target.AuthenticatedSite;
import org.eclipse.target.ITargetResource;
import org.eclipse.ui.IMemento;
import org.eclipse.webdav.client.*;
import org.eclipse.webdav.http.client.HttpClient;

public class WebDavSite extends AuthenticatedSite {

	/*
	 * Reuse HTTP client when creating DAV clients
	 */
	private static HttpClient httpClient = new HttpClient();
	
	public final static int DEFAULT_TIMEOUT = 60; // seconds
	
	private static final String HTTPCLIENT_CONNECTIONTIMEOUT_CTX = "httpClient.connectionTimeout"; //$NON-NLS-1$
	private static final String HTTPCLIENT_PROXYURL_CTX = "httpClient.proxyURL"; //$NON-NLS-1$
	
	private String proxy;
	private int timeout = DEFAULT_TIMEOUT;

	private DAVClient davClient;

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#save(org.eclipse.ui.IMemento)
	 */
	public void save(IMemento memento) {
		memento.putString(HTTPCLIENT_PROXYURL_CTX, proxy);
		memento.putInteger(HTTPCLIENT_CONNECTIONTIMEOUT_CTX, timeout);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#init(org.eclipse.ui.IMemento)
	 */
	protected void init(IMemento memento) {
		proxy = memento.getString(HTTPCLIENT_PROXYURL_CTX);
		Integer tempInt = memento.getInteger(HTTPCLIENT_CONNECTIONTIMEOUT_CTX);
		if (tempInt != null) {
			timeout = tempInt.intValue();
		}
		
		restoreAuthenticationInformation();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#getRootResource()
	 */
	public synchronized ITargetResource getRootResource() {
		if (davClient == null) {
			initializeDavClient();
		}
		return WebDavTargetResource.forSite(this, davClient);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#canBeReached(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean canBeReached(IProgressMonitor monitor) throws CoreException {
		return ((WebDavTargetResource)getRootResource()).canBeReached();
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimeout(int t) {
		timeout = t;
	}

	public void setProxyUrl(String text) {
		proxy = text;
	}
	
	private void initializeDavClient() {

		// Authentication.
		String username = getUsername();
		String password = getPassword();
		if ((username != null) && (password != null))
			httpClient.setAuthenticator(new Authenticator(username, password));

		// Proxy server.
		if (proxy != null && proxy.length() > 0) {
			try {
				httpClient.setDefaultProxyServerUrl(new URL(proxy));
			} catch (MalformedURLException exception) {
				// Log and continue without the proxy
				WebdavPlugin.log(WebdavPlugin.wrapException(exception));
			}
		}

		// Timeout converted to miliseconds.
		int timeout = getTimeout() * 1000;
		// connection recycler timeout and socket timeout being set to the same
		// value.
		httpClient.setConnectionTimeout(timeout);
		httpClient.setSoTimeout(timeout);

		// Can add other configs here later.
		davClient = new RemoteDAVClient(new WebDAVFactory(), httpClient);
	}

	public String getProxy() {
		return proxy;
	}

	public ResourceHandle getDavResource(URL url) {
		if (davClient == null) {
			initializeDavClient();
		}
		return WebDavTargetResource.createResource(this, davClient, url);
	}

}
